class HarmonicPoint extends Point {
	move() {
		const [p1, p2, p3] = this.pre;
		const new_point = harmonic_point_xy(p1, p2, p3);
		if (new_point) {
			this.exist = true;
			this.x = new_point.x;
			this.y = new_point.y;
		} else {
			this.exist = false;
		}
	}
}

function command_harmonic_point(x, y) {
	const point = get_object_at(x, y, points);
	if (point) {
		selected_objects.push(point);
	}
	if (selected_objects.length === 3) {
		const label = prompt('输入调和第四点的标签');
		if (label && !points.some(p => p.label === label)) {
			const [p1, p2, p3] = selected_objects;
			const {x, y} = harmonic_point_xy(p1, p2, p3);
			let harmonic_point = new HarmonicPoint(x, y, label);
			harmonic_point.pre = [p1, p2, p3];
			harmonic_point.free = false;
			harmonic_point.tikz = `\\tkzDefHarmonic[ext](${p1.label},${p2.label},${p3.label})\\tkzGetPoint{${label}}`;
			points.push(harmonic_point);
		} else {
			alert('标签是全局唯一的，不能重复');
		}
		selected_objects = [];
		draw();
	}
}

/**
 * 计算调和第四点的 xy 坐标
 * @param p1 点 p1
 * @param p2 点 p2
 * @param p3 点 p3
 * @return 返回点 p4 的坐标，满足 (p1,p2; p3,p4) 构成调和点列
 */
function harmonic_point_xy(p1, p2, p3) {
	if (!p1.exist || !p2.exist || !p3.exist) return null;
	const x1 = p1.x, y1 = p1.y;
	const x2 = p2.x, y2 = p2.y;
	const x3 = p3.x, y3 = p3.y;

	// 计算向量
	const dx12 = x2 - x1;
	const dy12 = y2 - y1;
	const dx13 = x3 - x1;
	const dy13 = y3 - y1;

	// 计算叉积
	const cross = dx12 * dy13 - dy12 * dx13;

	const t = ((dx13 * dx12 + dy13 * dy12) / (dx12 * dx12 + dy12 * dy12));

	// 计算一个调和点
	const x4a = x1 + (dx13 - 2 * t * dx12) / (1 - 2 * t);
	const y4a = y1 + (dy13 - 2 * t * dy12) / (1 - 2 * t);

	// 计算另一个调和点，通常选择这个
	const x4b = x1 - (dx13 - 2 * t * dx12) / (1 - 2 * t);
	const y4b = y1 - (dy13 - 2 * t * dy12) / (1 - 2 * t);
	return { 'x': x4b, 'y': y4b };
}
