class MirrorPoint extends Point {
	move() {
		const [point, segment] = this.pre;
		const new_point = mirror_point_xy(point, segment);
		if (new_point) {
			this.exist = true;
			this.x = new_point.x;
			this.y = new_point.y;
		} else {
			this.exist = false;
		}
	}
}

function command_mirror(x, y) {
	const obj = get_object_at(x, y, [...points, ...segments]);
	if (obj) {
		selected_objects.push(obj);
	}
	if (selected_objects.length === 2) {
		const [point, segment] = selected_objects;
		if (point instanceof Point && segment instanceof Segment) {
			const label = prompt('输入镜像点的标签');
			if (label && !points.some(p => p.label === label)) {
				const {x, y} = mirror_point_xy(point, segment);
				const mirror_point = new MirrorPoint(x, y, label);
				mirror_point.free = false;
				mirror_point.pre = [point, segment];
				mirror_point.tikz = `\\tkzDefPointBy[reflection = over ${segment.point1.label}--${segment.point2.label}](${point.label})\\tkzGetPoint{${label}}`;
				points.push(mirror_point);
			} else {
				alert('标签是全局唯一的，不能重复');
			}
		}
		selected_objects = [];
		draw();
	}
}


/**
 * 计算镜像点的 xy 坐标
 * @param point 被镜像的点
 * @param segment 镜像轴
 * @return 返回点 point 在镜像轴 segment 下的像的坐标
 */
function mirror_point_xy(point, segment) {
	if (!point.exist || !segment.point1.exist || !segment.point2.exist) return null;
	const {A, B, C} = segment.getABC();
	const x = ( (B * B - A * A) * point.x - 2 * A * B * point.y - 2 * A * C ) / ( A * A + B * B );
	const y = ( -2 * A * B * point.x + (A * A - B * B) * point.y - 2 * B * C ) / ( A * A + B * B );
	return {'x': x, 'y': y};
}
