class PerpFootPoint extends Point {
	move() {
		const [point, segment] = this.pre;
		const new_point = perp_foot_point_xy(point, segment);
		if (new_point) {
			this.exist = true;
			this.x = new_point.x;
			this.y = new_point.y;
		} else {
			this.exist = false;
		}
	}
}

function command_perp_foot(x, y) {
	const obj = get_object_at(x, y, [...points, ...segments]);
	if (obj) {
		selected_objects.push(obj);
	}
	if (selected_objects.length === 2) {
		const [point, segment] = selected_objects;
		if (point instanceof Point && segment instanceof Segment) {
			const label = prompt('输入垂足的标签');
			if (label && !points.some(p => p.label === label)) {
				const {x, y} = perp_foot_point_xy(point, segment);
				const perp_foot_point = new PerpFootPoint(x, y, label);
				perp_foot_point.pre = [point, segment];
				perp_foot_point.free = false;
				perp_foot_point.tikz = `\\tkzDefPointBy[projection= onto ${segment.point1.label}--${segment.point2.label}](${point.label})\\tkzGetPoint{${label}}`;
				points.push(perp_foot_point);
			} else {
				alert('标签是全局唯一的，不能重复');
			}
		}
		selected_objects = [];
		draw();
	}
}

/**
 * 计算垂足点的 xy 坐标
 * @param point 点 point
 * @param segment 线段 segment
 * @return 返回点 point 在线段 segment 所在直线上的垂足的坐标
 */
function perp_foot_point_xy(point, segment) {
	if (!point.exist || !segment.point1.exist || !segment.point2.exist) return null;
	const dx = segment.point2.x - segment.point1.x;
	const dy = segment.point2.y - segment.point1.y;
	const t = ((point.x - segment.point1.x) * dx + (point.y - segment.point1.y) * dy) / (dx * dx + dy * dy);
	const x = segment.point1.x + t * dx;
	const y = segment.point1.y + t * dy;
	return {'x': x, 'y': y};
}
