class TangentPoint extends Point {
	move() {
		const [point, circle, num] = this.pre;
		const new_point = tangent_point_xy(point, circle)[num];
		if (new_point) {
			this.exist = true;
			this.x = new_point.x;
			this.y = new_point.y;
		} else {
			this.exist = false;
		}
	}
}

function command_tangent(x, y) {
	const obj = get_object_at(x, y, [...points, ...circles]);
	if (obj) {
		selected_objects.push(obj);
	}
	if (selected_objects.length === 2) {
		const [obj1, obj2] = selected_objects;
		if (obj1 instanceof Point && obj2 instanceof Circle) {
			create_tangent(obj1, obj2);
		}
		selected_objects = [];
		draw();
	}
}

function create_tangent(point, circle) {
	const tangents = tangent_point_xy(point, circle);
	if (tangents[0]) {
		const label0 = prompt('输入切点0的标签');
		if (label0 && !points.some(p => p.label === label0)) {
			let tangent_point = new TangentPoint(tangents[0].x, tangents[0].y, label0);
			tangent_point.pre = [point, circle, 0];
			tangent_point.free = false;
			tangent_point.tikz = `\\tkzDefTangent[from=${point.label}](${circle.center.label},${circle.point.label})\\tkzGetFirstPoint{${label0}}`;
			points.push(tangent_point);
		} else {
			alert('标签是全局唯一的，不能重复');
		}
	}
	if (tangents[1]) {
		const label1 = prompt('输入交点1的标签');
		if (label1 && !points.some(p => p.label === label1)) {
			let tangent_point = new TangentPoint(tangents[1].x, tangents[1].y, label1);
			tangent_point.pre = [point, circle, 1];
			tangent_point.free = false;
			tangent_point.tikz = `\\tkzDefTangent[from=${point.label}](${circle.center.label},${circle.point.label})\\tkzGetSecondPoint{${label1}}`;
			points.push(tangent_point);
		} else {
			alert('标签是全局唯一的，不能重复');
		}
	}
}

/**
 * 计算点到圆所作切线的切点 xy 坐标
 * @param point 点 point
 * @param circle 圆 circle
 * @return 返回从点 point 作圆 circle 的切线，所得切点的坐标数组
 */
function tangent_point_xy(point, circle) {
	if (!point.exist || !circle.center.exist || !circle.point.exist) return null;
	const cx = circle.center.x;
	const cy = circle.center.y;
	const radius = Math.hypot(circle.point.x - circle.center.x, circle.point.y - circle.center.y);
	const px = point.x;
	const py = point.y;

	// 计算点到圆心的距离
	const d = Math.hypot(px - cx, py - cy);

	// 切点数组
	let tangents = [];

	// 如果距离小于半径，则没有切点
	if (d < radius) {
		return tangents;
	}

	// 如果距离等于半径，切点为点本身
	if (equal(d, radius)) {
		tangents.push({ 'x': px, 'y': py });
		return tangents;
	}

	// 如果距离大于半径，计算两个切点
	// 计算点与圆心的中点
	const mx = (px + cx) / 2;
	const my = (py + cy) / 2;
	// 计算旋转角度
	const rad = Math.asin(radius / d) * 2;
	// 将圆心绕中点顺时针、逆时针旋转 rad，得到切点
	const px1 = Math.cos(rad) * (cx - mx) - Math.sin(rad) * (cy - my) + mx;
	const py1 = Math.sin(rad) * (cx - mx) + Math.cos(rad) * (cy - my) + my;
	const px2 = Math.cos(-rad) * (cx - mx) - Math.sin(-rad) * (cy - my) + mx;
	const py2 = Math.sin(-rad) * (cx - mx) + Math.cos(-rad) * (cy - my) + my;

	tangents.push({ 'x': px1, 'y': py1 });
	tangents.push({ 'x': px2, 'y': py2 });

	return tangents;
}
