function get_object_at(x, y, objects) {
	const objs = objects.filter(obj => !obj.hidden);
	return objs.find(obj => {
		if (obj instanceof Point) {
			return xy_on_point(x, y, obj);
		} else if (obj instanceof Segment) {
			return xy_on_segment(x, y, obj);
		} else if (obj instanceof Circle) {
			return xy_on_circle(x, y, obj);
		}
		return false;
	});
}
/**
 * 判断鼠标点击的位置是否在点上
 * @param x 鼠标点击的 x 坐标
 * @param y 鼠标点击的 y 坐标
 * @param point canvas 上的点对象 point
 * @return 返回 (x,y) 位置是否是 point，用于判断是否选中点
 */
function xy_on_point(x, y, point) {
	return Math.hypot(point.x - x, point.y - y) < 10;
}
/**
 * 判断鼠标点击的位置是否在线段上
 * @param x 鼠标点击的 x 坐标
 * @param y 鼠标点击的 y 坐标
 * @param segment canvas 上的线段对象 segment
 * @return 返回 (x,y) 位置是否是 segment，用于判断是否选中线段
 */
function xy_on_segment(x, y, segment) {
	const dist1 = Math.hypot(x - segment.point1.x, y - segment.point1.y);
	const dist2 = Math.hypot(x - segment.point2.x, y - segment.point2.y);
	const dist = Math.hypot(segment.point2.x - segment.point1.x, segment.point2.y - segment.point1.y);
	return Math.abs(dist1 + dist2 - dist) < 10;
}
/**
 * 判断鼠标点击的位置是否在圆上
 * @param x 鼠标点击的 x 坐标
 * @param y 鼠标点击的 y 坐标
 * @param circle canvas 上的线段对象 circle
 * @return 返回 (x,y) 位置是否是 circle，用于判断是否选中圆
 */
function xy_on_circle(x, y, circle) {
	const dist = Math.hypot(x - circle.center.x, y - circle.center.y);
	const radius = Math.hypot(circle.point.x - circle.center.x, circle.point.y - circle.center.y);
	return Math.abs(dist - radius) < 10;
}
