const canvas = document.getElementById('canvas');
const ctx = canvas.getContext('2d');
ctx.translate(canvas.width / 2, canvas.height / 2);
ctx.scale(1, -1);
const hidden_list = document.getElementById('hidden-list');
const tkz_code = document.getElementById('code-output');

let points = [];
let segments = [];
let circles = [];
let hidden_objects = [];
let selected_objects = [];
let is_moving = false;
let move_target = null;
let current_tool = null;
let current_color = 'black';
const colors = ['black', 'blue', 'red', 'pink', 'purple', 'green'];

document.getElementById('move-btn').onclick = () => current_tool = 'move';
document.getElementById('hide-btn').onclick = () => current_tool = 'hide';
document.getElementById('delete-btn').onclick = () => current_tool = 'delete';
document.getElementById('label-btn').onclick = () => current_tool = 'label';
document.getElementById('position-btn').onclick = () => current_tool = 'position';
document.getElementById('coordinate-btn').onclick = () => current_tool = 'coordinate';
document.getElementById('generate-tkz-btn').onclick = () => generate_tikz();

document.getElementById('point-btn').onclick = () => current_tool = 'point';
document.getElementById('segment-btn').onclick = () => current_tool = 'segment';
document.getElementById('circle-btn').onclick = () => current_tool = 'circle';

document.getElementById('intersection-btn').onclick = () => current_tool = 'intersection';
document.getElementById('midpoint-btn').onclick = () => current_tool = 'midpoint';
document.getElementById('rotate-btn').onclick = () => current_tool = 'rotate';
document.getElementById('mirror-btn').onclick = () => current_tool = 'mirror';
document.getElementById('symmetry-btn').onclick = () => current_tool = 'symmetry';
document.getElementById('perp-foot-btn').onclick = () => current_tool = 'perp-foot';
document.getElementById('parallel-btn').onclick = () => current_tool = 'parallel';
document.getElementById('linear-btn').onclick = () => current_tool = 'linear';
document.getElementById('harmonic-btn').onclick = () => current_tool = 'harmonic';
document.getElementById('bisector-btn').onclick = () => current_tool = 'bisector';
document.getElementById('ex-bisector-btn').onclick = () => current_tool = 'ex-bisector';
document.getElementById('circum-center-btn').onclick = () => current_tool = 'circum-center';
document.getElementById('median-center-btn').onclick = () => current_tool = 'median-center';
document.getElementById('in-center-btn').onclick = () => current_tool = 'in-center';
document.getElementById('ex-center-btn').onclick = () => current_tool = 'ex-center';
document.getElementById('ortho-center-btn').onclick = () => current_tool = 'ortho-center';
document.getElementById('tangent-btn').onclick = () => current_tool = 'tangent';
// 初始化颜色选择工具
function init_color_picker() {
	const container = document.getElementById('color-picker');
	colors.forEach((color_code) => {
		const button = document.createElement('div');
		button.className = 'color-button';
		button.style.backgroundColor = color_code;
		button.addEventListener('click', () => {current_tool = 'color'; current_color = color_code});
		container.appendChild(button);
	});
}
init_color_picker();

function command_move(x, y) {
	const point = get_object_at(x, y, points);
	if (point) {
		is_moving = true;
		move_target = point;
		canvas.style.cursor = 'move';
	}
}

function command_hide(x, y) {
	const obj = get_object_at(x, y, [...points, ...segments, ...circles]);
	if (obj && !obj.hidden) {
		obj.hidden = true;
		update_hidden_list();
		draw();
	}
}

function command_color(x, y) {
	const obj = get_object_at(x, y, [...points, ...segments, ...circles]);
	console.log(obj);
	if (obj && !obj.hidden) {
		obj.color = current_color;
		draw();
	}
}

function command_delete(x, y) {
	// FIXIT
	const obj = get_object_at(x, y, [...points, ...segments, ...circles]);
	const objects = [...points, ...segments, ...circles];
	if (obj) {
		delete_dependent(obj, objects);
		draw();
	}
	
}

function clear_canvas() {
	// 保存当前转换矩阵
	ctx.save();
	// 重置转换矩阵到默认值
	ctx.setTransform(1, 0, 0, 1, 0, 0);
	// 清除画布
	ctx.clearRect(0, 0, canvas.width, canvas.height);
	// 恢复之前的转换矩阵
	ctx.restore();
}

function draw() {
	clear_canvas();
	points.forEach(point => draw_point(point));
	segments.forEach(segment => draw_segment(segment));
	circles.forEach(circle => draw_circle(circle));
}

function draw_point(point) {
	// 如果点被隐藏，或被移动到不存在的位置，则不绘制点
	if (point.hidden || !point.exist) return;
	ctx.strokeStyle = point.color;
	ctx.fillStyle = point.color;
	ctx.beginPath();
	ctx.arc(point.x, point.y, 5, 0, Math.PI * 2);
	ctx.fill();
	ctx.save();
	ctx.setTransform(1, 0, 0, 1, 0, 0);
	ctx.font = '12px Arial';
	ctx.textAlign = 'center';
	ctx.textBaseline = 'middle';
	let labelX = point.x;
	let labelY = point.y;

	switch (point.label_position) {
		case 'left':
			labelX -= 15;
			break;
		case 'right':
			labelX += 15;
			break;
		case 'above':
			labelY += 15;
			break;
		case 'below':
			labelY -= 15;
			break;
		case 'below left':
			labelX -= 10;
			labelY -= 10;
			break;
		case 'below right':
			labelX += 10;
			labelY -= 10;
			break;
		case 'above left':
			labelX -= 10;
			labelY += 10;
			break;
		case 'above right':
			labelX += 10;
			labelY += 10;
			break;
	}
	ctx.fillText(point.label, canvas.width / 2 + labelX, canvas.height / 2 - labelY);
	ctx.restore();
}

function draw_segment(segment) {
	if (segment.hidden) return;
	if (!segment.point1.exist || !segment.point2.exist) return;
	ctx.lineWidth = 3;
	ctx.strokeStyle = segment.color;
	ctx.beginPath();
	ctx.moveTo(segment.point1.x, segment.point1.y);
	ctx.lineTo(segment.point2.x, segment.point2.y);
	ctx.stroke();
}

function draw_circle(circle) {
	if (circle.hidden) return;
	if (!circle.center.exist || !circle.point.exist) return;
	const radius = Math.hypot(circle.point.x - circle.center.x, circle.point.y - circle.center.y);
	ctx.lineWidth = 3;
	ctx.strokeStyle = circle.color;
	ctx.beginPath();
	ctx.arc(circle.center.x, circle.center.y, radius, 0, Math.PI * 2);
	ctx.stroke();
}
// 刷新隐藏对象列表
function update_hidden_list() {
	hidden_list.innerHTML = '';
	points.forEach(point => {
		if (point.hidden) {
			const list_item = document.createElement('div');
			list_item.textContent = `点: ${point.label}`;
			list_item.onclick = () => {
				point.hidden = false;
				update_hidden_list();
				draw();
			}
			hidden_list.appendChild(list_item);
		}
	});
	segments.forEach(segment => {
		if (segment.hidden) {
			const list_item = document.createElement('div');
			list_item.textContent = `线段: ${segment.point1.label}${segment.point2.label}`;
			list_item.onclick = () => {
				segment.hidden = false;
				update_hidden_list();
				draw();
			}
			hidden_list.appendChild(list_item);
		}
	});
	circles.forEach(circle => {
		if (circle.hidden) {
			const list_item = document.createElement('div');
			list_item.textContent = `圆: ${circle.center.label}${circle.point.label}`;
			list_item.onclick = () => {
				circle.hidden = false;
				update_hidden_list();
				draw();
			}
			hidden_list.appendChild(list_item);
		}
	});
}

function generate_tikz() {
	let actions = [];
	let tkz_def_free_points = [];
	let tkz_def_unfree_points = [];
	let tkz_draw_points = new Map();
	let tkz_label_points = [];
	const free_points = points.filter(point => point.free);
	free_points.forEach(point => {
		tkz_def_free_points.push(`${point.x/100}/${point.y/100}/${point.label}`);
		if (!point.hidden) {
			let draw_points = tkz_draw_points.get(point.color);
			if (!draw_points) {
				draw_points = new Array();
			}
			draw_points.push(point.label);
			tkz_draw_points.set(point.color, draw_points);
			tkz_label_points.push(`\\tkzLabelPoints[${point.label_position}][color=${point.color}](${point.label})`);
		}
	});
	const unfree_points = points.filter(point => !point.free);
	unfree_points.forEach(point => {
		tkz_def_unfree_points.push(point.tikz);
		if (!point.hidden) {
			let draw_points = tkz_draw_points.get(point.color);
			if (!draw_points) {
				draw_points = new Array();
			}
			draw_points.push(point.label);
			tkz_draw_points.set(point.color, draw_points);
			tkz_label_points.push(`\\tkzLabelPoints[${point.label_position}][color=${point.color}](${point.label})`);
		}
	});
	//let tkz_draw_segments = [];
	let tkz_draw_segments = new Map();
	segments.forEach(segment => {
		if (!segment.hidden) {
			let draw_segments = tkz_draw_segments.get(segment.color);
			if (!draw_segments) {
				draw_segments = new Array();
			}
			draw_segments.push(`${segment.point1.label},${segment.point2.label}`);
			tkz_draw_segments.set(segment.color, draw_segments);
			//tkz_draw_segments.push(`${segment.point1.label},${segment.point2.label}`);
		}
	});
	//let tkz_draw_circles = [];
	let tkz_draw_circles = new Map();
	circles.forEach(circle => {
		if (!circle.hidden) {
			let draw_circles = tkz_draw_circles.get(circle.color);
			if (!draw_circles) {
				draw_circles = new Array();
			}
			draw_circles.push(`${circle.center.label},${circle.point.label}`);
			tkz_draw_circles.set(circle.color, draw_circles);
			//tkz_draw_circles.push(`${circle.center.label},${circle.point.label}`);
		}
	});
	actions.push('\\begin{tikzpicture}[scale=1,label style/.style={font=\\scriptsize}]');
	if (tkz_def_free_points.length > 0) {
		actions.push(`\\tkzDefPoints{${tkz_def_free_points.join(',')}}`);
	}
	if (tkz_def_unfree_points.length > 0) {
		actions.push(`${tkz_def_unfree_points.join('\n')}`);
	}
	tkz_draw_segments.forEach((segments, color) => {
		actions.push(`\\tkzDrawSegments[color=${color}](${segments.join(' ')})`);
	});
	tkz_draw_circles.forEach((circles, color) => {
		actions.push(`\\tkzDrawCircles[color=${color}](${circles.join(' ')})`);
	});
	/*
	if (tkz_draw_segments.length > 0) {
		actions.push(`\\tkzDrawSegments(${tkz_draw_segments.join(' ')})`);
	}
	if (tkz_draw_circles.length > 0) {
		actions.push(`\\tkzDrawCircles(${tkz_draw_circles.join(' ')})`);
	}
	*/
	actions.push(`${tkz_label_points.join('\n')}`);
	tkz_draw_points.forEach((points, color) => {
		actions.push(`\\tkzDrawPoints[color=${color},fill=${color}](${points.join(',')})`);
	});
	actions.push('\\end{tikzpicture}');
	tkz_code.value = actions.join('\n');
}

canvas.addEventListener('mousedown', (e) => {
	const rect = canvas.getBoundingClientRect();
	const x = -canvas.width / 2 + (e.clientX - rect.left);
	const y = canvas.height / 2 - (e.clientY - rect.top);
	if (current_tool === 'move') {
		command_move(x, y);
	}
});

canvas.addEventListener('mousemove', (e) => {
	if (is_moving && move_target) {
		const rect = canvas.getBoundingClientRect();
		const x = -canvas.width / 2 + (e.clientX - rect.left);
		const y = canvas.height / 2 - (e.clientY - rect.top);
		move_target.x = x;
		move_target.y = y;
		update_dependent_objects(move_target);
		draw();
	}
});

canvas.addEventListener('mouseup', () => {
	if (is_moving) {
		is_moving = false;
		move_target = null;
		canvas.style.cursor = 'default';
	}
});

canvas.addEventListener('click', (e) => {
	const rect = canvas.getBoundingClientRect();
	const x = -canvas.width / 2 + (e.clientX - rect.left);
	const y = canvas.height / 2 - (e.clientY - rect.top);
	if (current_tool !== 'move') {
		switch (current_tool) {
			case 'point':
				command_point(x, y);
				break;
			case 'segment':
				command_segment(x, y);
				break;
			case 'circle':
				command_circle(x, y);
				break;
			case 'intersection':
				command_inters(x, y);
				break;
			case 'hide':
				command_hide(x, y);
				break;
			case 'delete':
				command_delete(x, y);
				break;
			case 'label':
				command_label(x, y);
				break;
			case 'position':
				command_position(x, y);
				break;
			case 'coordinate':
				command_coordinate(x, y);
				break;
			case 'color':
				command_color(x, y);
				break;
			case 'midpoint':
				command_midpoint(x, y);
				break;
			case 'rotate':
				command_rotate(x, y);
				break;
			case 'mirror':
				command_mirror(x, y);
				break;
			case 'symmetry':
				command_symmetry(x, y);
				break;
			case 'perp-foot':
				command_perp_foot(x, y);
				break;
			case 'parallel':
				command_parallel(x, y);
				break;
			case 'linear':
				command_linear_point(x, y);
				break;
			case 'harmonic':
				command_harmonic_point(x, y);
				break;
			case 'bisector':
				command_bisector(x, y);
				break;
			case 'ex-bisector':
				command_ex_bisector(x, y);
				break;
			case 'circum-center':
				command_circum_center(x, y);
				break;
			case 'median-center':
				command_median_center(x, y);
				break;
			case 'in-center':
				command_in_center(x, y);
				break;
			case 'ex-center':
				command_ex_center(x, y);
				break;
			case 'ortho-center':
				command_ortho_center(x, y);
				break;
			case 'tangent':
				command_tangent(x, y);
				break;
		}
	}
});

// 查找所有依赖于 obj 的对象，并将它们删除
function delete_dependent(obj, objects) {
	// 存储所有依赖于 obj 的对象
	const dependents = new Set();
	dependents.add(obj);
	function dfs(obj) {
		for (let other of objects) {
			if (other.pre && other.pre.includes(obj) && !dependents.has(other)) {
				dependents.add(other);
				dfs(other);
			}
		}
	}

	// 从 obj 开始查找依赖
	dfs(obj);
	// 删除所有依赖于 obj 的对象
	points = points.filter(point => !dependents.has(point));
	segments = segments.filter(segment => !dependents.has(segment));
	circles = circles.filter(circle => !dependents.has(circle));
}

function update_dependent_objects(point) {
	for (point of points) {
		if (!point.free) {
			point.move();
		}
	}
}
